
let max_int = 0x3fffffff
let state = ref 1
let decal = ref 99733

let init n =
  let n = if n mod 2 = 0 then (n + 1) land max_int else n in
  decal := n * 99733;
  state := n

let int p =
  let n = (99901 * !state + !decal) land max_int in
  state := n;
  n mod p

let float f =
  let i = int max_int in
  f *. float i /. float max_int

let color () =
  let red = float 0.75 +. 0.25 in
  let blue = float 0.75 +. 0.125 in
  let green = float (0.75 -. blue) +. 0.125 in
  (red,green,blue,1.0)

(*
  Tests:

  Cycle lengt is always maximum 2^31
  Passes equiprobability tests
  Fails all dependency tests


let look_for_cycle p =
  init p;
  let n0 = !state in
  let rec fn i =
    let n1 = (99901 * !state + 99733) land max_int in
    state := n1;
    if i mod 10_000_000 = 0 then (Printf.printf "\r%d" i; flush stdout);
    if n1 = n0 then
      (Printf.printf "\rcycle after %d for seed %d\n" i p; flush stdout)
    else
      fn (i+1)
  in
  fn 0


(* Return the sum of the squares of v[i0,i1[ *)
let rec sumsq v i0 i1 =
  if i0 >= i1 then 0.0
  else if i1 = i0 + 1 then Pervasives.float v.(i0) *. Pervasives.float v.(i0)
  else sumsq v i0 ((i0+i1)/2) +. sumsq v ((i0+i1)/2) i1
;;

let chisquare g n r =
  if n <= 10 * r then invalid_arg "chisquare";
  let f = Array.make r 0 in
  for i = 1 to n do
    let t = g r in
    f.(t) <- f.(t) + 1
  done;
(*
  for i = 0 to r - 1 do
    Printf.printf "%d: %d " i f.(i)
  done;
*)
  let t = sumsq f 0 r
  and r = Pervasives.float r
  and n = Pervasives.float n in
  let sr = 2.0 *. sqrt r in
  let mi = r -. sr in
  let mc = (r *. t /. n) -. n in
  let ma = r +. sr in
  Printf.printf "%f <= %f <= %f\n" mi mc ma; flush stdout;
  if not (mi <= mc && mc <= ma) then print_endline "FAILED";
  mi, mc, ma
;;

(* This is to test for linear dependencies between successive random numbers.
*)
let st = ref 0;;
let init_diff r = st := int r;;
let diff r =
  let x1 = !st
  and x2 = int r
  in
  st := x2;
  if x1 >= x2 then
    x1 - x2
  else
    r + x1 - x2
;;

let st1 = ref 0
and st2 = ref 0
;;

(* This is to test for quadratic dependencies between successive random
   numbers.
*)
let init_diff2 r = st1 := int r; st2 := int r;;
let diff2 r =
  let x1 = !st1
  and x2 = !st2
  and x3 = int r
  in
  st1 := x2;
  st2 := x3;
  (x3 - x2 - x2 + x1 + 2*r) mod r
;;

init 27182818; chisquare int 100_000 2;;
init 27182818; chisquare int 100_000 1000;;
init 27182818; chisquare int 100_000 100;;
init 27182818; chisquare int 100_000 5000;;
init 27182818; chisquare int 100_000 1000;;
init 27182818; chisquare int 100_000 1024;;
init 299792643; chisquare int 100_000 1024;;
init 14142136; chisquare int 100_000 1024;;
init 0; chisquare int 100_0000 1024;;
init 27182818; init_diff 1024; chisquare diff 100_0000 1024;;
init 0; init_diff 1024; chisquare diff 100_0000 1024;;
init 27182818; init_diff 100; chisquare diff 100_0000 100;;
init 0; init_diff 100; chisquare diff 100_0000 100;;
init 27182818; init_diff2 1024; chisquare diff2 100_000 1024;;
init 27182818; init_diff2 100; chisquare diff2 100_000 100;;
init 14142136; init_diff2 100; chisquare diff2 100_000 100;;
init 299792643; init_diff2 100; chisquare diff2 100_000 100;;

let _ =
  for i = 0 to 100 do
    look_for_cycle (2*i+1)
  done

*)
